package com.diagbot.config.security;

import com.diagbot.facade.TokenFacade;
import com.diagbot.util.HttpUtils;
import com.diagbot.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.AccessDecisionManager;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.authentication.AccountExpiredException;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Collection;

/**
 * @Description: 自定义权限拦截
 * @author: gaodm
 * @time: 2018/8/23 13:46
 */
@Service
public class UrlAccessDecisionManager implements AccessDecisionManager {
    @Autowired
    private TokenFacade tokenFacade;

    @Override
    public void decide(Authentication authentication, Object object, Collection<ConfigAttribute> configAttributes) throws AccessDeniedException, InsufficientAuthenticationException {
        HttpServletRequest request = ((FilterInvocation) object).getHttpRequest();
        String url, method;
        if (matchPermitAllUrl(request)) {
            return;
        }
        if ("anonymousUser".equals(authentication.getPrincipal())) {
            throw new AccessDeniedException("no right");
        } else {
            String tokenStr = HttpUtils.getHeaders(request).get("Authorization");
            if (StringUtil.isNotEmpty(tokenStr)) {
                tokenStr = tokenStr.replaceFirst("Bearer ", "");
                Boolean res = tokenFacade.verifyToken(tokenStr, 1);
                if (!res) {
                    throw new AccountExpiredException("token expire");
                }
            }
            for (GrantedAuthority ga : authentication.getAuthorities()) {
                String[] authority = ga.getAuthority().split(";");
                url = authority[0];
                method = authority[1];
                if (matchers(url, request)) {
                    if (method.equals(request.getMethod()) || "ALL".equals(method)) {
                        return;
                    }
                }
            }
        }
        throw new AccessDeniedException("no right");
    }


    @Override
    public boolean supports(ConfigAttribute attribute) {
        return true;
    }

    @Override
    public boolean supports(Class<?> clazz) {
        return true;
    }

    private Boolean matchPermitAllUrl(HttpServletRequest request) {
        if (matchers("/swagger/**", request)
                || matchers("/v2/**", request)
                || matchers("/swagger-ui.html/**", request)
                || matchers("/swagger-resources/**", request)
                || matchers("/webjars/**", request)
                || matchers("/druid/**", request)
                || matchers("/actuator/**", request)
                || matchers("/hystrix/**", request)
                || matchers("/sys/user/getJwt", request)
                || matchers("/sys/user/refreshJwt", request)
                || matchers("/sys/user/checkToken", request)
                //|| matchers("/sys/user/getUserOrgMenu", request)
                || matchers("/oauth/token", request)
                || matchers("/oauth/check_token", request)
                //|| matchers("/tran/lisConfig/isExistRecord", request)
                //|| matchers("/tran/lisConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/lisConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/lisConfig/deleteRecord", request)
                //|| matchers("/tran/lisConfig/deleteRecords", request)
                || matchers("/tran/lisConfig/getPage", request)
                // || matchers("/tran/lisConfig/importExcel", request)
                // || matchers("/tran/lisConfig/exportExcel", request)
                || matchers("/tran/lisConfig/exportExcelModule", request)
                // || matchers("/tran/pacsConfig/isExistRecord", request)
                //|| matchers("/tran/pacsConfig/saveOrUpdateRecord", request)
                // || matchers("/tran/pacsConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/pacsConfig/deleteRecord", request)
                //|| matchers("/tran/pacsConfig/deleteRecords", request)
                || matchers("/tran/pacsConfig/getPage", request)
                // || matchers("/tran/pacsConfig/importExcel", request)
                //|| matchers("/tran/pacsConfig/exportExcel", request)
                || matchers("/tran/pacsConfig/exportExcelModule", request)
                //|| matchers("/tran/diseaseConfig/isExistRecord", request)
                //|| matchers("/tran/diseaseConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/diseaseConfig/saveOrUpdateRecords", request)
                //|| matchers("/tran/diseaseConfig/deleteRecord", request)
                //|| matchers("/tran/diseaseConfig/deleteRecords", request)
                || matchers("/tran/diseaseConfig/getPage", request)
                //|| matchers("/tran/diseaseConfig/importExcel", request)
                //|| matchers("/tran/diseaseConfig/exportExcel", request)
                || matchers("/tran/diseaseConfig/exportExcelModule", request)
                //|| matchers("/tran/drugConfig/isExistRecord", request)
                //|| matchers("/tran/drugConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/drugConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/drugConfig/deleteRecord", request)
                //|| matchers("/tran/drugConfig/deleteRecords", request)
                || matchers("/tran/drugConfig/getPage", request)
                //|| matchers("/tran/drugConfig/importExcel", request)
                //|| matchers("/tran/drugConfig/exportExcel", request)
                || matchers("/tran/drugConfig/exportExcelModule", request)
                // || matchers("/tran/operationConfig/isExistRecord", request)
                // || matchers("/tran/operationConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/operationConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/operationConfig/deleteRecord", request)
                //|| matchers("/tran/operationConfig/deleteRecords", request)
                || matchers("/tran/operationConfig/getPage", request)
                //|| matchers("/tran/operationConfig/importExcel", request)
                //|| matchers("/tran/operationConfig/exportExcel", request)
                || matchers("/tran/operationConfig/exportExcelModule", request)
                // || matchers("/tran/deptConfig/isExistRecord", request)
                // || matchers("/tran/deptConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/deptConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/deptConfig/deleteRecord", request)
                //|| matchers("/tran/deptConfig/deleteRecords", request)
                || matchers("/tran/deptConfig/getPage", request)
                // || matchers("/tran/deptConfig/importExcel", request)
                // || matchers("/tran/deptConfig/exportExcel", request)
                || matchers("/tran/deptConfig/exportExcelModule", request)
                // || matchers("/tran/transfusionConfig/isExistRecord", request)
                // || matchers("/tran/transfusionConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/transfusionConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/transfusionConfig/deleteRecord", request)
                //|| matchers("/tran/transfusionConfig/deleteRecords", request)
                || matchers("/tran/transfusionConfig/getPage", request)
                // || matchers("/tran/transfusionConfig/importExcel", request)
                // || matchers("/tran/transfusionConfig/exportExcel", request)
                || matchers("/tran/transfusionConfig/exportExcelModule", request)
                //|| matchers("/tran/scaleConfig/isExistRecord", request)
                //|| matchers("/tran/scaleConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/scaleConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/scaleConfig/deleteRecord", request)
                //|| matchers("/tran/scaleConfig/deleteRecords", request)
                || matchers("/tran/scaleConfig/getPage", request)
                // || matchers("/tran/scaleConfig/importExcel", request)
                // || matchers("/tran/scaleConfig/exportExcel", request)
                || matchers("/tran/scaleConfig/exportExcelModule", request)
                //|| matchers("/tran/nurseConfig/isExistRecord", request)
                //|| matchers("/tran/nurseConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/nurseConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/nurseConfig/deleteRecord", request)
                //|| matchers("/tran/nurseConfig/deleteRecords", request)
                || matchers("/tran/nurseConfig/getPage", request)
                // || matchers("/tran/nurseConfig/importExcel", request)
                // || matchers("/tran/nurseConfig/exportExcel", request)
                || matchers("/tran/nurseConfig/exportExcelModule", request)
                //|| matchers("/tran/tcmdiseaseConfig/isExistRecord", request)
                //|| matchers("/tran/tcmdiseaseConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/tcmdiseaseConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/tcmdiseaseConfig/deleteRecord", request)
                //|| matchers("/tran/tcmdiseaseConfig/deleteRecords", request)
                || matchers("/tran/tcmdiseaseConfig/getPage", request)
                // || matchers("/tran/tcmdiseaseConfig/importExcel", request)
                // || matchers("/tran/tcmdiseaseConfig/exportExcel", request)
                || matchers("/tran/tcmdiseaseConfig/exportExcelModule", request)
                //|| matchers("/tran/tcmsyndromeConfig/isExistRecord", request)
                //|| matchers("/tran/tcmsyndromeConfig/saveOrUpdateRecord", request)
                //|| matchers("/tran/tcmsyndromeConfig/saveOrUpdateRecords", request)
                // || matchers("/tran/tcmsyndromeConfig/deleteRecord", request)
                //|| matchers("/tran/tcmsyndromeConfig/deleteRecords", request)
                || matchers("/tran/tcmsyndromeConfig/getPage", request)
                // || matchers("/tran/tcmsyndromeConfig/importExcel", request)
                // || matchers("/tran/tcmsyndromeConfig/exportExcel", request)
                || matchers("/tran/mappingConfig/exportExcelModule", request)
                || matchers("/tran/mappingConfig/precDataMatch", request)
                || matchers("/tran/mappingConfig/importExcel", request)
                || matchers("/tran/mappingConfig/exportExcel", request)
                || matchers("/tran/mappingConfig/getPage", request)
                || matchers("/tran/mappingConfig/getRecord", request)
                || matchers("/tran/mappingConfig/getRelatedMapping", request)
                || matchers("/tran/mappingConfig/isExistRecord", request)
                || matchers("/tran/mappingConfig/saveOrUpdateRecord", request)
                || matchers("/tran/mappingConfig/deleteRecord", request)
                || matchers("/tran/mappingConfig/deleteRecords", request)
                //|| matchers("/tran/hospitalInfo/saveRecord", request)
                || matchers("/tran/hospitalInfo/getHospitalInfo", request)
                || matchers("/tran/hospitalInfo/getAllHospitalInfo", request)
                || matchers("/tran/hospitalInfo/getAllEnableHospitalInfo", request)
                || matchers("/tran/hospitalInfo/getHospitalInfoById", request)
                || matchers("/sys/versionInfo/getVersionInfoAlls", request)
                || matchers("/sys/disclaimerInfo/getDisclaimerInfo", request)
                || matchers("/sys/mr/createMr", request)
                || matchers("/sys/mr/getMr", request)
                || matchers("/sys/mr/getTcmMr", request)
                || matchers("/sys/mr/getIndicationMr", request)
                || matchers("/sys/plan/getSysPlanInfoDatas", request)
                || matchers("/sys/mrqc/analyze_run", request)
                || matchers("/sys/tokenPermission/delPermission", request)
                || matchers("/sys/tokenPermission/getPermission", request)
                || matchers("/sys/push/push", request)
                || matchers("/sys/push/pushApi", request)
                || matchers("/sys/push/indicationPush", request)
                || matchers("/sys/push/indicationExtPush", request)
                || matchers("/sys/push/pushPlan", request)
                || matchers("/demo/templateInfo/updateByIdUsNames", request)
                || matchers("/demo/templateInfo/saveTemplateInfo", request)
                || matchers("/demo/templateInfo/cancelTemplateInfos", request)
                || matchers("/demo/templateInfo/getTemplatePageAlls", request)
                || matchers("/demo/templateInfo/getTemplatebyId", request)
                || matchers("/sys/plan/getPlanInfoPages", request)
                /*|| matchers("/sys/plan/savePlanInfoDatas", request)
                || matchers("/sys/plan/getSysPlanInfoDatas", request)
                || matchers("/sys/plan/cancelPlanDatas", request)*/
                //|| matchers("/sys/plan/revStopPlans", request)
                || matchers("/sys/tokenHospital/getTokenHospital", request)
                || matchers("/demo/retrieval/index", request)
                || matchers("/kl/conceptInfo/staticKnowledgeIndex", request)
                || matchers("/kl/conceptInfo/staticKnowledgeIndexWithoutInfo", request)
                || matchers("/kl/conceptInfo/getStaticKnowledge", request)
                || matchers("/kl/conceptInfo/getStaticKnowledgeForHIS", request)
                || matchers("/kl/conceptInfo/getPage", request)
                || matchers("/kl/conceptInfo/staticIndexPage", request)
                //|| matchers("/kl/conceptInfo/saveOrUpdateRecord", request)
                //|| matchers("/kl/conceptInfo/changeStatus", request)
                || matchers("/kl/conceptInfo/isExist", request)
                || matchers("/kl/conceptInfo/getRecordById", request)
                || matchers("/kl/conceptInfo/updateHasInfoStatusBatch", request)
                || matchers("/sys/planDetail/getPlanDetailDatas", request)
                /*|| matchers("/sys/planDetail/savePlanDetails", request)
                || matchers("/sys/planDetail/cancelPlanDetails", request)
                || matchers("/sys/planDetail/revStopPlanDetails", request)
                || matchers("/sys/plan/getDefaultPlans", request)*/
                || matchers("/sys/dictionaryInfo/getList", request)
                || matchers("/sys/dictionaryInfo/getListBack", request)
                || matchers("/sys/plan/getPlanInfoIds", request)
                || matchers("/sys/file/uploadImage", request)
                || matchers("/sys/file/deleteRemoteFile", request)
                || matchers("/sys/mrqc/caseWritingPrompt", request)
                || matchers("/demo/testcaseInfo/testcaseProcess", request)
                || matchers("/demo/mrtestInfo/importExcel", request)
                || matchers("/demo/mrtestInfo/exportExcel", request)
                || matchers("/demo/mrtestInfo/mrTestProcess", request)
                || matchers("/kl/dictionary/getDictionarys", request)
                || matchers("/klRulePlan/getMenu", request)
                || matchers("/klRule/getKlRuleInfoPage", request)
                || matchers("/klRule/getByIdRuleInfo", request)
                || matchers("/klRule/saveRuleInfo", request)
                || matchers("/klRule/clearRuleInfo", request)
                || matchers("/klRule/disableRuleInfo", request)
                || matchers("/klRule/startRuleInfo", request)
                || matchers("/klDisease/searchConcept", request)
                || matchers("/cache/clearRuleAll", request)
                || matchers("/term/termMatching", request)
                || matchers("/", request)) {
            return true;
        }
        return false;
    }

    private boolean matchers(String url, HttpServletRequest request) {
        AntPathRequestMatcher matcher = new AntPathRequestMatcher(url);
        if (matcher.matches(request)) {
            return true;
        }
        return false;
    }
}